open Gtk;;
open Glib;;
open Gtk.Unsafe;;

class gtkobject (gtkobject : Gtk.Unsafe.gtkobject) =
object
  val gtkobject = gtkobject
  method get_gtkobject = gtkobject
  method disconnect_slot n = signal_disconnect gtkobject n
  method destroy = object_destroy gtkobject
end

class widget (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit gtkobject gtkobject
  method show = widget_show gtkobject
  method hide = widget_hide gtkobject
  method set_tip tip_text tip_private = 
    tooltips_set_tip (tooltips_new ()) gtkobject tip_text tip_private
  method realize = widget_realize gtkobject 
  method unrealize = widget_unrealize gtkobject 
  method set_usize x y = widget_set_usize gtkobject x y
  method window_of = window_of gtkobject
  method style_of = style_of gtkobject
  method set_events events = widget_set_events gtkobject events

  method connect_expose (func: int->int->int->int->bool) =
    signal_connect gtkobject
    "expose_event" (fun [Pointer ptr] ->
      let x = Gdk.Event.Extract.expose_x ptr
      and y = Gdk.Event.Extract.expose_y ptr
      and w = Gdk.Event.Extract.expose_w ptr
      and h = Gdk.Event.Extract.expose_h ptr
      in Bool (func x y w h))
      
  method connect_key_press_event (func: Glib.Pointer.t -> bool) =
    signal_connect gtkobject
      "key_press_event" (fun [Pointer ptr] -> Bool (func ptr))
  (* UNFINISHED *)
end

(* 5.13 Container widget *)
class container (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit widget gtkobject
  method border_width = container_border_width gtkobject
  method add (child : widget) = container_add gtkobject (child #get_gtkobject)
  method remove (child : widget) = container_remove gtkobject (child #get_gtkobject)
  method disable_resize = container_disable_resize gtkobject
  method enable_resize = container_enable_resize gtkobject
  method block_resize = container_block_resize gtkobject
  method unblock_resize = container_unblock_resize gtkobject
  method need_resize = container_need_resize gtkobject
  (* UNFINISHED difficult to make foreach type-safe *)
  method focus = container_focus gtkobject
  (* UNFINISHED difficult to make children type-safe *)
end

(* 5.4 Bin widget *)
class bin (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit container gtkobject
end

(* 5.1 Alignment widget *)
class alignment (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit bin gtkobject
  method set = alignment_set gtkobject
end

(* 5.42 Misc widget *) (* UNFINISHED *)
class misc (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit widget gtkobject
end

(* 5.2 Arrow widget *)
class arrow (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit misc gtkobject
  method set = arrow_set gtkobject
end

(* 5.23 Frame widget *) (* UNFINISHED *)
class frame (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit bin gtkobject
  method set_shadow_type = frame_set_shadow_type gtkobject
end

(* 5.3 Aspect frame widget *)
class aspect_frame (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit frame gtkobject
  method set = aspect_frame_set gtkobject
end

(* 5.5 Box widget *)
class box (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit container gtkobject
  method pack_start (child : widget) =
    box_pack_start gtkobject (child #get_gtkobject)
  method pack_end (child : widget) =
    box_pack_end gtkobject (child #get_gtkobject)
  method pack_start_defaults (child : widget) =
    box_pack_start_defaults gtkobject (child #get_gtkobject)
  method pack_end_defaults (child : widget) =
    box_pack_end_defaults gtkobject (child #get_gtkobject)
  method set_homogeneous = box_set_homogeneous gtkobject
  method set_spacing = box_set_spacing gtkobject
  method reorder_child (child : widget) =
    box_reorder_child gtkobject (child #get_gtkobject)
  method query_child_packing (child : widget) =
    box_query_child_packing gtkobject (child #get_gtkobject)
end

(* 5.6 Button box widget *)
class button_box (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit box gtkobject
  method set_child_size = button_box_set_child_size gtkobject
  method set_child_ipadding = button_box_set_child_ipadding gtkobject
  method set_layout = button_box_set_layout gtkobject
  method get_spacing = button_box_get_spacing gtkobject
  method get_child_size = button_box_get_child_size gtkobject
  method get_child_ipadding = button_box_get_child_ipadding gtkobject
  method get_layout = button_box_get_layout gtkobject
end

(* ### 5.7 Button widget *)
class button (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit container gtkobject
  (* signals completed *)
  method connect_clicked (func : unit -> unit) =
    signal_connect gtkobject "clicked" (fun _ -> func (); Unit)
  method connect_pressed (func : unit -> unit) =
    signal_connect gtkobject "pressed" (fun _ -> func (); Unit)
  method connect_enter (func : unit -> unit) =
    signal_connect gtkobject "enter" (fun _ -> func (); Unit)
  method connect_leave (func : unit -> unit) =
    signal_connect gtkobject "leave" (fun _ -> func (); Unit)

  method pressed = button_pressed gtkobject
  method released = button_released gtkobject
  method clicked = button_clicked gtkobject
  method enter = button_enter gtkobject
  method leave = button_leave gtkobject
end

(* ### 5.60 Toggle button widget *) (* UNFINISHED *)
class toggle_button (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit button gtkobject
  method set_state flag = toggle_button_set_state gtkobject flag
end

(* ### 5.8 Check button widget *)
class check_button (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit toggle_button gtkobject
end

class item (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit bin gtkobject
end

class menu_shell (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit container gtkobject
end

class menu (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit menu_shell gtkobject
  method append (item : menu_item) =
    menu_append gtkobject (item #get_gtkobject)
end

and menu_item (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit item gtkobject
  method set_submenu (menu : menu) =
    menu_item_set_submenu gtkobject (menu #get_gtkobject)
  method connect_activate (f:unit->unit) =
    signal_connect 
      gtkobject 
      "activate" 
      (fun _ -> f () ; Unit) 
  method select () = menu_item_select gtkobject
  method deselect () = menu_item_deselect gtkobject
end

(* ### 5.9 Check menu item widget *)
class check_menu_item (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit menu_item gtkobject
  method set_state = check_menu_item_set_state gtkobject
  method toggled = check_menu_item_toggled gtkobject
  method connect_toggled (func : unit -> unit) =
    signal_connect gtkobject "toggled" (fun _ -> func (); Unit)
end

(* ### 5.10 Compound list *)
class clist (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit container gtkobject
  method set_border = clist_set_border gtkobject
  method set_selection_mode = clist_set_selection_mode gtkobject
  method set_policy = clist_set_policy gtkobject
  method freeze = clist_freeze gtkobject
  method thaw = clist_thaw gtkobject
  method column_titles_show = clist_column_titles_show gtkobject
  method column_titles_hide = clist_column_titles_hide gtkobject
  method set_column_title = clist_set_column_title gtkobject
  method set_column_widget i (w : widget)= clist_set_column_widget gtkobject i
    (w #get_gtkobject)
  method set_column_justification = clist_set_column_justification gtkobject
  method set_column_width = clist_set_column_width gtkobject
  method set_row_height = clist_set_row_height gtkobject
  method set_row_moveto = clist_moveto gtkobject
  method set_text = clist_set_text gtkobject
(* clist_set_pixmap $$$ *)
(* clist_setpixtext $$$ *)
(* clist_set_foreground $$$ *)
(* clist_set_background $$$ *)
  method set_shift = clist_set_shift gtkobject
(* clist_append $$$ *)
(* clist_insert $$$ *)
  method remove_row = clist_remove gtkobject
(* clist_set_row_data $$$ *)
(* clist_get_row_data $$$ *)
  method select_row = clist_select_row gtkobject
  method unselect_row = clist_unselect_row gtkobject
  method connect_select_row (func : int->int->unit) = 
    signal_connect gtkobject (* UNFINISHED *)
      "select_row" (fun [Int row; Int column; _] -> func row column; Unit)
  method connect_unselect_row (func : int->int->unit) = 
    signal_connect gtkobject (* UNFINISHED *)
      "unselect_row" (fun [Int row; Int column; _] -> func row column; Unit)
  method connect_click_column (func : int->unit) =
    signal_connect gtkobject
      "click_column" (fun [Int column] -> func column; Unit)
end

let clist_new i = new clist (clist_new i)

(* ### 5.65 Vertical box widget *)
class vbox (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit box gtkobject
end

(* ### 5.11 Color selector *)
class color_selection (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit vbox gtkobject
  method set_update_policy= color_selection_set_update_policy gtkobject
  method set_color = color_selection_set_color gtkobject
  method get_color = color_selection_get_color gtkobject
  method set_opacity = color_selection_set_opacity gtkobject
  method connect_color_changed (func : unit->unit) =
    signal_connect gtkobject "color_changed" (fun _ -> func (); Unit)
end

class drawing_area (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit widget gtkobject
  method size width height = drawing_area_size gtkobject width height
end

and scrolled_window (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit container gtkobject
end

and paned (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit widget gtkobject
  method add1 (child : widget) =
    paned_add1 gtkobject (child #get_gtkobject)
  method add2 (child : widget) =
    paned_add2 gtkobject (child #get_gtkobject)
  method handle_size size = paned_handle_size gtkobject size
  method gutter_size size = paned_gutter_size gtkobject size
end

and vpaned (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit paned gtkobject
end

and hpaned (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit paned gtkobject
end

and list_item (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit item gtkobject
end

and window (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit bin gtkobject
  method connect_delete_event (func : unit->bool) =
    signal_connect gtkobject "delete_event" (fun _ -> Bool(func ()))
  method connect_move_resize (func : (*int ref->int ref->*)int->int->bool) =
    signal_connect gtkobject "move_resize"
      (fun [(*(Pointer xp); (Pointer yp) ;*)_;_; (Int w); (Int h)] ->
         (*let x=ref (Pointer.int_peek xp) and y=ref (Pointer.int_peek yp)
         in let rv=func x y w h
            in Pointer.int_poke xp !x; Pointer.int_poke yp !y;
               Bool rv*) Bool(func w h))
  method set_title s = window_set_title gtkobject s
end

and hbox (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit box gtkobject
end

and radio_button (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit check_button gtkobject
end

and list_box (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit container gtkobject
  method insert_items (items: list_item list) index =
    list_insert_items gtkobject
      (GList.from_ml (List.map (fun item-> item #get_gtkobject) items))
      index
  method append_items (items: list_item list) =
    list_append_items gtkobject
      (GList.from_ml (List.map (fun item-> item #get_gtkobject) items))
end

class radio_menu_item (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit check_menu_item gtkobject
end

and menu_bar (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit menu_shell gtkobject
  method append (item : menu_item) =
    menu_bar_append gtkobject (item #get_gtkobject)
end

and label (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit misc gtkobject
end

and progress_bar (gtkobject : Gtk.Unsafe.gtkobject) = 
object
  inherit widget gtkobject
  method update ratio = progress_bar_update gtkobject ratio
end

and separator (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit widget gtkobject
end

and hseparator (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit separator gtkobject
end

and vseparator (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit separator gtkobject
end

and editable (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit widget gtkobject
  method insert_text text len pos = editable_insert_text gtkobject text len pos
  method select_region start endd = editable_select_region gtkobject start endd
  method get_chars start endd = editable_get_chars gtkobject start endd
  method get_text = editable_get_chars gtkobject 0 (-1)
end

and text (gtkobject : Gtk.Unsafe.gtkobject) =
object (t)
  inherit editable gtkobject
  method insert_substring font s start length = text_insert gtkobject font s start length
  method insert font s = t#insert_substring font s 0 (-1)
  method set_editable flag = text_set_editable gtkobject flag
  method forward_delete n = text_forward_delete gtkobject n
  method backward_delete n = text_backward_delete gtkobject n
end

and entry (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit editable gtkobject
  method connect_activate (func : unit -> unit) =
    signal_connect gtkobject "activate" (fun _ -> func (); Unit)
  method set_text text = entry_set_text gtkobject text 
  method get_text = entry_get_text gtkobject
end

and notebook (gtkobject : Gtk.Unsafe.gtkobject) =
object
  inherit widget gtkobject
  method append_page (page:widget) (label:widget) = 
    notebook_append_page gtkobject (page #get_gtkobject) (label #get_gtkobject)
  method prepend_page (page:widget) (label:widget) = 
    notebook_prepend_page gtkobject (page #get_gtkobject) (label #get_gtkobject)
  method insert_page (page:widget) (label:widget) pos = 
    notebook_insert_page gtkobject (page #get_gtkobject) (label #get_gtkobject) pos
  method remove_page (page:widget) (label:widget) pos = 
    notebook_remove_page gtkobject (page #get_gtkobject) (label #get_gtkobject) pos
  method set_page pos = 
    notebook_set_page gtkobject pos
  method next_page = 
    notebook_next_page gtkobject 
  method prev_page = 
    notebook_prev_page gtkobject 
  method set_tab_pos pos = notebook_set_tab_pos gtkobject pos
  method set_show_tabs show_tabs = notebook_set_show_tabs gtkobject show_tabs		
  method set_show_border show_border = notebook_set_show_border gtkobject show_border		
end;;

let frame_new s = new frame (frame_new s)

let scrolled_window_new () = new scrolled_window (scrolled_window_new ())

let notebook_new () = new notebook (notebook_new ()) 

let text_new () = new text (text_new ()) ;; 

let entry_new () = new entry (entry_new ())
let entry_new_with_max_length n = 
  if n>=0 & n<=65535 
  then new entry (entry_new_with_max_length n)
  else raise (Invalid_argument "entry_new_with_max_length")

let vpaned_new () = new vpaned(vpaned_new()) ;;

let hpaned_new () = new vpaned(hpaned_new()) ;;

let window_new window_type = new window (window_new window_type);;

let button_new_with_label s = new button (button_new_with_label s);;
let check_button_new_with_label s = new check_button (check_button_new_with_label s);;
let toggle_button_new_with_label s = new toggle_button (toggle_button_new_with_label s);;

let list_item_new_with_label s = new list_item (list_item_new_with_label s);;
let list_new () = new list_box (list_new ());;

let map_fold f org =
  let rec zorglub =
    function
    	[] -> org, []
      | h::t ->
	  let now, rest = zorglub t in
            let after, point = f now h in
              after, (point::rest)
  in function l -> snd (zorglub l);;

let radio_buttons_new_with_labels = 
  map_fold (fun group label ->
    let widget = radio_button_new_with_label group label in
      (Some (radio_button_group widget)), (new radio_button widget)) None;;

let rec radio_buttons_set_index = function
    [] -> invalid_arg "GtkObj.radio_buttons_set_index"
  | (h : radio_button)::t -> function
        0 -> h #set_state true
      |	n -> h #set_state false;
             radio_buttons_set_index t (pred n);; 

let hbox_new homogeneous spacing = new hbox (hbox_new homogeneous spacing);;
let vbox_new homogeneous spacing = new vbox (vbox_new homogeneous spacing);; 

let color_selection_new title = new color_selection
  (color_selection_new title);;

let menu_new () = new menu (menu_new ());;
let menu_bar_new () = new menu_bar (menu_bar_new ());;
let menu_item_new_with_label s = new menu_item (menu_item_new_with_label s);;
let menu_item_new ()=
  new menu_item (menu_item_new ());;
let check_menu_item_new_with_label s = new check_menu_item (check_menu_item_new_with_label s);;
let check_menu_item_new () = new check_menu_item (check_menu_item_new ());;
(*let radio_menu_item_new_with_label s = new radio_menu_item (radio_menu_item_new_with_label s);;*)

let label_new s = new label (label_new s);;

let progress_bar_new () = new progress_bar (progress_bar_new ());;

let hseparator_new () = new hseparator (hseparator_new ());;
let vseparator_new () = new vseparator (vseparator_new ());;

let drawing_area_new () = new drawing_area (drawing_area_new ());;

let ( << ) x y = x#add (y :> widget); x;;




